###########################################################################
#
# ImagePlug.pm -- simple text plugin
# A component of the Greenstone digital library software
# from the New Zealand Digital Library Project at the 
# University of Waikato, New Zealand.
#
# Copyright (C) 1999 New Zealand Digital Library Project
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
###########################################################################

package ImagePlug;

use BasPlug;

sub BEGIN {
    @ISA = ('BasPlug');
}

sub new {
    my ($class) = @_;
    my $self = new BasPlug ("ImagePlug", @_);

    return bless $self, $class;
}

sub get_default_process_exp {
    my $self = shift (@_);

    return q^(?i)(\.jpe?g|\.gif|\.png|\.tiff?)$^; 
}

sub run_convert {
    my $self = shift (@_);
    my $filename = shift (@_);
    my $file = shift (@_);
    my $doc_obj = shift (@_);

    my $type = "unknown";

    if (-e "$filename.thumbnail.gif") {
	print "ImagePlug: both \"$filename\" and \"$filename.thumbnail.gif\" exist.\n";
	return undef;
    }

    my $result = "";
    $result = `convert -verbose -loop 1 -delay 1 -geometry 100x100 $filename $filename.thumbnail.gif` ;
    my $section = $doc_obj->get_top_section();

    if ($result =~ m/=>.* ([0-9]+)x([0-9]+)=>([0-9]+)x([0-9]+)/) {
	$doc_obj->add_metadata ($section, "ImageHeight", $1);
	$doc_obj->add_metadata ($section, "ImageWidth", $2);
	$doc_obj->add_metadata ($section, "ThumbHeight", $3);
	$doc_obj->add_metadata ($section, "ThumbWidth", $4);
    }
    my $size = "";
    if ($result =~ m/^[^\n]* ([0-9]+)b/) {
	$size = $1;
    }
    if ($result =~ m/^[^\n]* ([0-9]+)kb/) {
	$size = 1024 * $1;
    }

    if ($result =~ m/^[^\n]*JPE?G/) {
	$type = "jpeg";
    }
    if ($result =~ m/^[^\n]*GIF/) {
	$type = "gif";
    }
    if ($result =~ m/^[^\n]*PNG/) {
	$type = "png";
    }
    if ($result =~ m/^[^\n]*TIFF?/) {
	$type = "tiff";
    }
    
    if ($type ne "unknown") {
	$doc_obj->add_metadata ($section, "ImageType", $type);
    }
    $doc_obj->add_metadata ($section, "ThumbType", "gif");
    $doc_obj->add_metadata ($section, "Image", "$file");
    $doc_obj->add_metadata ($section, "Thumb", "$file.thumbnail.gif");
    if ($size != 0) {
	$doc_obj->add_metadata ($section, "ImageSize", $size);
    }
    print $result;
    return $type;
}


# The ImagePlug read() function. This function does all the right things
# to make general options work for a given plugin. It calls the process()
# function which does all the work specific to a plugin (like the old
# read functions used to do). Most plugins should define their own 
# process() function and let this read() function keep control.
#
# ImagePlug overrides read() because there is no need to read the actual 
# text of the file in, because the contents of the file is not text...
#
# Return number of files processed, undef if can't process
# Note that $base_dir might be "" and that $file might 
# include directories

sub read {
    my $self = shift (@_);
    my ($pluginfo, $base_dir, $file, $metadata, $processor, $maxdocs) = @_;

    my $filename = &util::filename_cat($base_dir, $file);
    return 0 if $self->{'block_exp'} ne "" && $filename =~ /$self->{'block_exp'}/;
    if ($filename !~ /$self->{'process_exp'}/ || !-f $filename) {
	return undef;
    }
    my $plugin_name = ref ($self);
    $file =~ s/^[\/\\]+//; # $file often begins with / so we'll tidy it up
    
    # create a new document
    my $doc_obj = new doc ($filename, "indexed_doc");
    
    #run convert to get the thumbnail and extract size and type info
    my $mimetype = run_convert($self, $filename, $file, $doc_obj);

    #add the image as an associated file ...
    if (-e "$filename.thumbnail.gif") {
	$doc_obj->associate_file("$filename.thumbnail.gif","$file.thumbnail.gif","image/gif",$section);
    } else {
	print "ImagePlug: couldn't find \"$filename.thumbnail.gif\"\n";
    }

    if (defined $mimetype) {
	$doc_obj->associate_file($filename,$file,"image/$mimetype",$section);
    }

    #create an empty text string so we don't break downstream plugins 
    my $text = "";

    # include any metadata passed in from previous plugins 
    # note that this metadata is associated with the top level section
    $self->extra_metadata ($doc_obj, $section, $metadata);

    # do plugin specific processing of doc_obj
    return undef unless defined ($self->process (\$text, $pluginfo, $base_dir, $file, $metadata, $doc_obj));

    # do any automatic metadata extraction
    $self->auto_extract_metadata ($doc_obj);

    # add an OID
    $doc_obj->set_OID();

#    $doc_obj->add_text($section, "<pre>\n$text\n</pre>");

    # process the document
    $processor->process($doc_obj);

    if (-e "$filename.thumbnail.gif") {
      util::rm("$filename.thumbnail.gif");
    }

    return 1; # processed the file
}

# do plugin specific processing of doc_obj
sub process {
    my $self = shift (@_);
    my ($textref, $pluginfo, $base_dir, $file, $metadata, $doc_obj) = @_;
    my $outhandle = $self->{'outhandle'};
    
    return 1;
}

1;











